@@ -58,7 +58,7 @@ pub mod prelude {
         error::{Error as _, Simple},
         text::TextParser as _,
         span::Span as _,
-        primitive::{any, end, filter, filter_map, just, one_of, none_of, seq},
+        primitive::{any, end, filter, filter_map, just, one_of, none_of, seq, take_until},
         recovery::{skip_then_retry_until, nested_delimiters},
         recursive::recursive,
         text,
@@ -215,7 +215,7 @@ pub trait Parser<I: Clone, O> {
 
     /// Parse a stream of tokens, yielding an output *or* any errors that were encountered along the way.
     ///
-    /// If you wish to attempt to produce an output even if errors are encountered, use `Parser::parse_recovery`.
+    /// If you wish to attempt to produce an output even if errors are encountered, use [`Parser::parse_recovery`].
     fn parse<
         'a,
         Iter: Iterator<Item = (I, <Self::Error as Error<I>>::Span)> + 'a,
@@ -238,13 +238,12 @@ pub trait Parser<I: Clone, O> {
         Debug(self, Rc::new(x), *std::panic::Location::caller())
     }
 
-    /// Map the output of this parser to aanother value.
+    /// Map the output of this parser to another value.
     ///
     /// # Examples
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
     /// #[derive(Debug, PartialEq)]
     /// enum Token { Word(String), Num(u64) }
     ///
@@ -267,7 +266,7 @@ pub trait Parser<I: Clone, O> {
 
     /// Map the output of this parser to another value, making use of the pattern's overall span.
     ///
-    /// This is very useful when generating an AST but you need to know what span to give each AST node.
+    /// This is very useful when generating an AST that attaches a span to each AST node.
     fn map_with_span<U, F: Fn(O, <Self::Error as Error<I>>::Span) -> U>(self, f: F) -> MapWithSpan<Self, F, O>
         where Self: Sized
     { MapWithSpan(self, f, PhantomData) }
@@ -287,7 +286,6 @@ pub trait Parser<I: Clone, O> {
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
     /// let byte = text::int::<_, Simple<char>>(10)
     ///     .try_map(|s, span| s
     ///         .parse::<u8>()
@@ -315,7 +313,6 @@ pub trait Parser<I: Clone, O> {
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
     /// let frac = text::digits(10)
     ///     .chain(just('.'))
     ///     .chain::<char, _, _>(text::digits(10))
@@ -337,7 +334,6 @@ pub trait Parser<I: Clone, O> {
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
     /// #[derive(Clone, Debug, PartialEq)]
     /// enum Op { Add, Sub, Mul, Div }
     ///
@@ -351,13 +347,14 @@ pub trait Parser<I: Clone, O> {
     /// ```
     fn to<U: Clone>(self, x: U) -> To<Self, O, U> where Self: Sized { To(self, x, PhantomData) }
 
-    /// Left-fold the output of the parser into a single value, where the output is of type `(_, Vec<_>)`.
+    /// Left-fold the output of the parser into a single value.
+    ///
+    /// The output of the original parser must be of type `(A, impl IntoIterator<Item = B>)`.
     ///
     /// # Examples
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
     /// let int = text::int::<char, Cheap<char>>(10)
     ///     .map(|s| s.parse().unwrap());
     ///
@@ -368,18 +365,21 @@ pub trait Parser<I: Clone, O> {
     /// assert_eq!(sum.parse("1+12+3+9"), Ok(25));
     /// assert_eq!(sum.parse("6"), Ok(6));
     /// ```
-    fn foldl<A, B, F: Fn(A, B) -> A>(self, f: F) -> Foldl<Self, F, A, B>
+    fn foldl<A, B, F: Fn(A, B::Item) -> A>(self, f: F) -> Foldl<Self, F, A, B>
     where
-        Self: Parser<I, (A, Vec<B>)> + Sized
+        Self: Parser<I, (A, B)> + Sized,
+        B: IntoIterator,
     { Foldl(self, f, PhantomData) }
 
-    /// Right-fold the output of the parser into a single value, where the output is of type `(Vec<_>, _)`.
+    /// Right-fold the output of the parser into a single value.
+    ///
+    /// The output of the original parser must be of type `(impl IntoIterator<Item = A>, B)`. Because right-folds work
+    /// backwards, the iterator must implement [`DoubleEndedIterator`] so that it can be reversed.
     ///
     /// # Examples
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
     /// let int = text::int::<char, Cheap<char>>(10)
     ///     .map(|s| s.parse().unwrap());
     ///
@@ -393,23 +393,24 @@ pub trait Parser<I: Clone, O> {
     /// assert_eq!(signed.parse("-17"), Ok(-17));
     /// assert_eq!(signed.parse("--+-+-5"), Ok(5));
     /// ```
-    fn foldr<'a, A, B, F: Fn(A, B) -> B + 'a>(self, f: F) -> Foldr<Self, F, A, B>
+    fn foldr<'a, A, B, F: Fn(A::Item, B) -> B + 'a>(self, f: F) -> Foldr<Self, F, A, B>
     where
-        Self: Parser<I, (Vec<A>, B)> + Sized
+        Self: Parser<I, (A, B)> + Sized,
+        A: IntoIterator,
+        A::IntoIter: DoubleEndedIterator,
     { Foldr(self, f, PhantomData) }
 
     /// Ignore the output of this parser, yielding `()` as an output instead.
     ///
     /// This can be used to reduce the cost of parsing by avoiding unnecessary allocations (most collections containing
-    /// [ZSTs](https://doc.rust-lang.org/nomicon/exotic-sizes.html#zero-sized-types-zsts) do
-    /// [not allocate](https://doc.rust-lang.org/std/vec/struct.Vec.html#guarantees)). For example, it's common to want
-    /// to ignore whitespace in many grammars.
+    /// [ZSTs](https://doc.rust-lang.org/nomicon/exotic-sizes.html#zero-sized-types-zsts)
+    /// [do not allocate](https://doc.rust-lang.org/std/vec/struct.Vec.html#guarantees)). For example, it's common to
+    /// want to ignore whitespace in many grammars (see [`text::whitespace`]).
     ///
     /// # Examples
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
     /// // A parser that parses any number of whitespace characters without allocating
     /// let whitespace = filter::<_, _, Cheap<char>>(|c: &char| c.is_whitespace())
     ///     .ignored()
@@ -420,21 +421,22 @@ pub trait Parser<I: Clone, O> {
     /// ```
     fn ignored(self) -> Ignored<Self, O> where Self: Sized { To(self, (), PhantomData) }
 
-    /// Collect the output of this parser into a collection.
+    /// Collect the output of this parser into a type implementing [`FromIterator`].
     ///
-    /// This is commonly useful for collecting [`Vec<char>`] outputs into [`String`]s.
+    /// This is commonly useful for collecting [`Vec<char>`] outputs into [`String`]s, or [`(T, U)`] into a
+    /// [`HashMap`] and is analagous to [`Iterator::collect`].
     ///
     /// # Examples
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
-    /// let word = filter::<_, _, Cheap<char>>(|c: &char| c.is_alphabetic())
-    ///     .repeated()
-    ///     .collect::<String>();
+    /// let word = filter::<_, _, Cheap<char>>(|c: &char| c.is_alphabetic()) // This parser produces an output of `char`
+    ///     .repeated() // This parser produces an output of `Vec<char>`
+    ///     .collect::<String>(); // But `Vec<char>` is less useful than `String`, so convert to the latter
     ///
     /// assert_eq!(word.parse("hello"), Ok("hello".to_string()));
     /// ```
+    // TODO: Make `Parser::repeated` generic over an `impl FromIterator` to reduce required allocations
     fn collect<C: core::iter::FromIterator<O::Item>>(self) -> Map<Self, fn(O) -> C, O>
         where Self: Sized, O: IntoIterator
     { self.map(|items| C::from_iter(items.into_iter())) }
@@ -445,7 +447,6 @@ pub trait Parser<I: Clone, O> {
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
     /// let word = filter::<_, _, Cheap<char>>(|c: &char| c.is_alphabetic())
     ///     .repeated().at_least(1)
     ///     .collect::<String>();
@@ -462,7 +463,6 @@ pub trait Parser<I: Clone, O> {
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
     /// let int = just('-').or_not()
     ///     .chain(filter::<_, _, Cheap<char>>(|c: &char| c.is_ascii_digit() && *c != '0')
     ///         .chain(filter::<_, _, Cheap<char>>(|c: &char| c.is_ascii_digit()).repeated()))
@@ -507,7 +507,6 @@ pub trait Parser<I: Clone, O> {
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
     /// let zeroes = filter::<_, _, Cheap<char>>(|c: &char| *c == '0').ignored().repeated();
     /// let digits = filter(|c: &char| c.is_ascii_digit()).repeated();
     /// let integer = zeroes
@@ -528,7 +527,6 @@ pub trait Parser<I: Clone, O> {
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
     /// let word = filter::<_, _, Cheap<char>>(|c: &char| c.is_alphabetic())
     ///     .repeated().at_least(1)
     ///     .collect::<String>();
@@ -560,7 +558,6 @@ pub trait Parser<I: Clone, O> {
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
     /// let ident = text::ident::<_, Simple<char>>()
     ///     .padded_by(just('!'));
     ///
@@ -573,15 +570,12 @@ pub trait Parser<I: Clone, O> {
         where Self: Sized
     { other.clone().ignore_then(self).then_ignore(other) }
 
-    // fn then_catch(self, end: I) -> ThenCatch<Self, I> where Self: Sized { ThenCatch(self, end) }
-
     /// Parse the pattern surrounded by the given delimiters.
     ///
     /// # Examples
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
     /// // A LISP-style S-expression
     /// #[derive(Debug, PartialEq)]
     /// enum SExpr {
@@ -630,7 +624,6 @@ pub trait Parser<I: Clone, O> {
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
     /// let op = just::<_, Cheap<char>>('+')
     ///     .or(just('-'))
     ///     .or(just('*'))
@@ -657,7 +650,6 @@ pub trait Parser<I: Clone, O> {
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
     /// #[derive(Debug, PartialEq)]
     /// enum Expr {
     ///     Error,
@@ -670,7 +662,7 @@ pub trait Parser<I: Clone, O> {
     ///     .delimited_by('[', ']')
     ///     .map(Expr::List)
     ///     // If parsing a list expression fails, recover at the next delimiter, generating an error AST node
-    ///     .recover_with(nested_delimiters('[', ']', [], || Expr::Error))
+    ///     .recover_with(nested_delimiters('[', ']', [], |_| Expr::Error))
     ///     .or(text::int(10).map(Expr::Int))
     ///     .padded());
     ///
@@ -684,7 +676,7 @@ pub trait Parser<I: Clone, O> {
     /// // Additionally, the AST we get back still has useful information.
     /// assert_eq!(ast, Some(Expr::List(vec![Expr::Error, Expr::Error])));
     /// ```
-    fn recover_with<S: Strategy<I, O>>(self, strategy: S) -> Recovery<Self, S> where Self: Sized {
+    fn recover_with<S: Strategy<I, O, Self::Error>>(self, strategy: S) -> Recovery<Self, S> where Self: Sized {
         Recovery(self, strategy)
     }
 
@@ -696,7 +688,6 @@ pub trait Parser<I: Clone, O> {
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
     /// let word = filter::<_, _, Cheap<char>>(|c: &char| c.is_alphabetic())
     ///     .repeated().at_least(1)
     ///     .collect::<String>();
@@ -712,13 +703,12 @@ pub trait Parser<I: Clone, O> {
     /// Parse an expression any number of times (including zero times).
     ///
     /// Input is eagerly parsed. Be aware that the parser will accept no occurences of the pattern too. Consider using
-    /// [`Parser::repeated().at_least`] instead if it better suits your use-case.
+    /// [`Repeated::at_least`] instead if it better suits your use-case.
     ///
     /// # Examples
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
     /// let num = filter::<_, _, Cheap<char>>(|c: &char| c.is_ascii_digit())
     ///     .repeated().at_least(1)
     ///     .collect::<String>()
@@ -740,7 +730,6 @@ pub trait Parser<I: Clone, O> {
     ///
     /// ```
     /// # use chumsky::{prelude::*, error::Cheap};
-    ///
     /// let shopping = text::ident::<_, Simple<char>>()
     ///     .padded()
     ///     .separated_by(just(','));
@@ -765,6 +754,10 @@ pub trait Parser<I: Clone, O> {
     ///
     /// Boxing a parser might be useful for:
     ///
+    /// - Dynamically building up parsers at runtime
+    ///
+    /// - Improving compilation times (Rust can struggle to compile code containing very long times)
+    ///
     /// - Passing a parser over an FFI boundary
     ///
     /// - Getting around compiler implementation problems with long types such as
